<?php	// Setup and installation script

# title     : Setup and installation
# version   : 1.02
# author    : Markavian
# last edit : 23/11/2009
# function  : One stop html page for configuration for quick installation of a site
# contact   : code@mkv25.net

// Example script:
/* ------------------------
include('setup.class.php');

// Variables for the application //
$setup = new SetupScript('constants.php', 'pe5ches4', 'Setup and configuration');
// Note: If a value is required, set the default value to false to force validation on the variable
// Note: $setup->AddVariable('name', max_length, default_value, 'helpful_description', 'helpful_example');
// Note: $setup->AddInstruction('MethodName', 'Description of function');

$setup->AddVariable('MYSQL_SERVER', 0, false, false, 'MySQL server', 'localhost:/tmp/mysql5.sock');
$setup->AddVariable('MYSQL_DATABASE', 0, false, false, '- Database', 'database');
$setup->AddVariable('MYSQL_USERNAME', 0, false, false, '- User', 'user');
$setup->AddVariable('MYSQL_PASSWORD', 0, true, false, ' - Password', 'password');
$setup->AddLinebreak();

$setup->AddVariable('COMMENTS_TABLE', 0, false, false, ' Table name to store entries', 'oc_comments');
$setup->AddVariable('SERVICE_DIRECTORY', 0, false, false, ' URL of OpenComments Service', 'http://mkv25.net/opencomments/v2/');
$setup->AddLinebreak();

$setup->AddVariable('BBCODE_SITE_ROOT', 0, false, false, ' Default for bbcode links', 'http://mkv25.net/info/');
$setup->AddLinebreak();

$setup->AddVariable('TRUSTED_USER_1', 0, false, false, '', 'http://mkv25.net/openid/Markavian');
$setup->AddVariable('TRUSTED_USER_2', 0, false, false, '', 'http://localhost/openid/Markavian');

// End variables //
// Post setup instructions //
$setup->AddInstruction('CreateTables', 'Create tables in the database needed for the application', true);
$setup->AddInstruction('AnnounceSuccess', 'Annouce successful installation');
// End instructions //

function CreateTables()
{
	echo '<h2>SQL Tables</h2>';
	echo '<p>Creating tables...</p>';
	
	require_once('constants.php');
	require_once('../config/sql_class.php');
	$sql = new sql($MYSQL_SERVER, $MYSQL_DATABASE, $MYSQL_USERNAME, $MYSQL_PASSWORD);

	$instructions = array();
	
	$instructions[] = sprintf("
CREATE TABLE IF NOT EXISTS `%s` (
  `comment_id` int(11) NOT NULL auto_increment,
  `url` varchar(255) NOT NULL,
  `comment` text NOT NULL,
  `openid` tinytext NOT NULL,
  `username` varchar(32) NOT NULL,
  `website` varchar(128) NOT NULL,
  `postdate` datetime NOT NULL,
  `comment_type` enum('comment','feedback','todo','bug','suggestion') NOT NULL,
  PRIMARY KEY  (`comment_id`,`url`)
) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;
	", $COMMENTS_TABLE);
	
	foreach($instructions as $name=>$query)
	{
		$sql->query($query, $name);
	}
	
	echo '<p>Sparing error messages listed above; tables created.</p>';
}

function AnnounceSuccess()
{
	echo "<h2>Success</h2>";
	echo "<p>Installation was a success.</p>";
}

// Create, process, and display page //
$setup->DisplayPage();
// ------------------------ */
// End of example


class SetupScript
{
	var $filename;
	var $variables;
	var $instructions;
	var $safeword;
	
	var $title;
	var $description;
	var $footer;
	
	function SetupScript($filename, $safeword, $title='Setup and configuration', $description='', $footer='')
	{
		$this->filename = $filename;
		$this->variables = array();
		$this->instructions = array();
		$this->safeword = $safeword;
		
		$this->title = $title;
		$this->description = ($description) ? $description : 'This installation script will create the file '.$this->filename.'.';
		$this->footer = $footer = ($footer) ? $footer : 'Setup and configuration script; created by <a href="http://mkv25.net/info/">John Beech</a>';
	}
	
	function AddVariable($name, $maxlength=0, $default=false, $description='', $example='')
	{
		if(is_a($name, 'SetupVariable'))
		{
			$var = $name;
		}
		else
		{
			 $var = new SetupVariable($name, $maxlength, $default, $description, $example);
		}
		$this->variables[$name] = $var;
	}
	
	function AddLinebreak()
	{
		$this->variables[] .= 'linebreak';
	}
	
	function AddInstruction($name, $description, $optional=false)
	{
		if(is_a($name, 'SetupInstruction'))
		{
			$instruction = $name;
		}
		else
		{
			$instruction = new SetupInstruction($name, $description, $optional);
		}	
		$this->instructions[$name] = $instruction;
	}
	
	function DisplayPage()
	{
		echo SetupTemplate::createHTML($this->title, $this->CreatePage(), $this->footer);
	}

	function CreatePage()
	{
		$nl = "\n";
		$output = '';
		
		/* Display common header; title, optional or default description */
		$output .= '<h1>' . $this->title . '</h1>'.$nl;
		$output .= '<p>' . $this->description . '</p>';
		
		/* Check if form has been submitted */
		if($_POST['setupForm_submit'] == 'Save Values')
		{
			/* Validate submitted data  */
			$errors = 0;
			$error = '';
			
			if($_REQUEST['frm_safeword'] != $this->safeword)
			{
				$errors++;
				$error .= SetupScript::CreateMessage('error', 'The safe word is incorrect- cannot continue.');
			}
			
			foreach($this->variables as $key=>$variable)
			{
				if($variable == 'linebreak')
				{
					// skip
				}
				else
				{
					$userValue = $_POST['frm_'.$variable->name];
					$variable->userValue = stripslashes(trim($userValue));
					
					if($variable->validate() == false)
					{
						$errors++;
						$error .=SetupScript::CreateMessage('warning', $variable->validationError);
					}
				}
			}
			
			if($errors > 0)
			{
				/* Display errors */
				$output .= '<div class="errors">'.$nl;
				$output .= $error;
				$output .= '</div>'.$nl;
				
				/* Display form again so user can edit errors */
				$output .= $this->CreateForm();
			}
			else
			{
				/* No errors, save variables */
				$this->SaveVariables();
				$num_vars = count($this->variables);
				$output .= SetupScript::CreateMessage('success', 'Form variables saved ('.$num_vars.' total); you can now include <a href="'.$this->filename.'">'.$this->filename.'</a> in your project.');
				
				$output .= $this->ProcessInstructions();
				
				$output .= SetupScript::CreateMessage('success', 'Optional: <a href="">Click here to reset the script</a>- useful for checking the variables have saved correctly.');
				$output .= SetupScript::CreateMessage('warning', 'Recommended: Delete this script from your server to prevent risk of unauthorised access');
			}
		}
		else
		{
			/* Display default form */
			$output .= $this->ReadVariables();
			$output .= $this->CreateForm();
		}
		return $output;
	}
	
	function ReadVariables()
	{
		$output = '';
		
		if(file_exists($this->filename))
		{
			include($this->filename);
			$output .= SetupScript::CreateMessage('warning', 'The file <b>'.$this->filename.' already exists</b>, saving this form will overwrite the following values:');
		}
		else
		{
			return;
		}
			
		foreach($this->variables as $key=>$variable)
		{
			if($variable == 'linebreak')
			{
				// skip
			}
			else
			{
				$variable->userValue = ${$variable->name};
			}
		}
		
		return $output;
	}
	
	function SaveVariables()
	{
		$nl = "\n";
		$date = date('l jS \of F Y h:i:s A');
		
		/* Output introduction and file information */
		$output = '<?php

// WARNING: This is a dynamically created file; edit only as a last resort
// Generated by SetupScript class on '.$date.' 
// - Local path:  ' . $_SERVER['SCRIPT_FILENAME'] . '
// - Remote path: http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] . '
// Dependencies: setup.class.php

// Title:				'.$this->title.'
// Description:	'.$this->description.'
// Configuration variables:

';
		/* Output script variables */
		foreach($this->variables as $key=>$variable)
		{
			if($variable == 'linebreak')
			{
				$output .= $nl;
			}
			else
			{
				$comment = (trim($variable->description) != '') ? ' // '.$variable->description : '';
				$output .= '$'.$variable->name.' = "'.$variable->userValue.'";'.$comment.$nl;
			}
		}
		$output .= $nl;
		
		/* Output list of setup instructions, for reference */
		$output .= '// Additionally, the following instructions were executed: ' . $nl;
		$count = 0;
		foreach($this->instructions as $key=>$instruction)
		{
			$count++;
			$info = ($instruction->optional) ? ' (optional)' : '';
			$output .= '// ' . $count . '.	' . $instruction->description . $info . $nl;
		}
		$output .= ($count) ? '' : '// None' . $nl;
		
		/* Close output */
		$output .= $nl.'?>';
		
		/* Write file */
		$fp = fopen($this->filename, "w");
		fwrite($fp, $output);
		fclose($fp);
	}
	
	function ProcessInstructions()
	{
		$nl = "\n";
		$output = '';
		
		foreach($this->instructions as $key=>$instruction)
		{
			$function = $instruction->functionName;
			$input_name = 'frm_option_'.$instruction->functionName;
			
			$checked = ($_REQUEST[$input_name] == 'true') ? true : false;
			if($checked)
			{
				ob_start();
				$function();
				$instruction->executed = true;
				
				$output .= ob_get_clean() . $nl;
				$output .= SetupScript::CreateMessage('success', 'Processed: ' . $instruction->description);
			}
			else
			{
				$output .= SetupScript::CreateMessage('warning', 'Skipped: ' . $instruction->description);
			}
		}
		return $output;
	}
	
	function CreateForm()
	{
		// Add safeword field
		$this->variables['safeword'] = new SetupVariable('safeword', 32, true, false, 'Safe word used to activate this script');
		$this->variables['safeword']->userValue = $_REQUEST['frm_safeword'];
		$this->AddLinebreak();
		
		$nl = "\n";
		$output = '<form name="setupForm" class="setupForm" action="" method="POST">'.$nl;
		
		$output .= '<table border="0" cellspacing="5" cellpadding="0">'.$nl;
		foreach($this->variables as $key=>$variable)
		{
			if($variable == 'linebreak')
			{
				$output .= '<tr class="linebreak"><td colspan="2" class="linebreak">&nbsp;</td></tr>' . $nl;
			}
			else
			{
				$output .= '<tr>'.$nl;
				$output .= '	<th class="variableName">'.ucfirst($variable->name).'</th>'.$nl;
				$output .= '	<td>'.$this->CreateFormInput($variable).'</td>'.$nl;
				$output .= '</tr>'.$nl;
			}
		}
		
		$output .= '<tr><td colspan="2">' . $this->ListInstructions() . '</td></tr>' . $nl;
		
		$output .= '<tr><td align="right" colspan="2" class="actionButtons">' . $nl;
		$output .= '	<input name="setupForm_cancel" value="Cancel" type="submit" />' . $nl;
		$output .= '  <input name="setupForm_submit" value="Save Values" type="submit" />' . $nl;
		$output .= '</td></tr>' . $nl;
		$output .= '</table>';

		
		$output .= '</form>'.$nl;
		
		return $output;
	}
	
	function ListInstructions()
	{
		$nl = "\n";
		$output = '';
		
		if(count($this->instructions) > 0)
		{
			$output .= '<p>After saving the above values, the following instructions will be called:</p>'.$nl;
			$output .= '<ol class="listOfInstructions">'.$nl;
			foreach($this->instructions as $key=>$instruction)
			{
				$output .= '	<li>'.$this->CreateInstructionOption($instruction).'</li>'.$nl;
			}
			$output .= '</ol>'.$nl;
		}
		return $output;
	}
	
	function CreateFormInput($variable)
	{
		$output = '';
		if(is_a($variable, 'SetupVariable'))
		{
			$inputname = 'frm_'.$variable->name;
			$default = ($variable->default != false) ? ' <i class="defaultValue">Default ('.$variable->default.')</i>' : '';
			$userValue = ($variable->userValue != '') ? $variable->userValue : $variable->default;
			if($variable->maxlength < 255)
			{
				$maxlength = ($variable->maxlength > 0) ? ' maxlength="'.$variable->maxlength.'"' : '';
				$type = ($variable->passworded) ? 'password' : 'text';
				
				$output .= '<input name="'.$inputname.'"'.$maxlength.' type="'.$type.'" class="text '.$variable->name.'" value="'.$userValue.'" />';
			}
			else
			{
				$output .= '<textarea name="frm_'.$inputname.'"'.$maxlength.' class="text '.$variable->name.'" >'.$userValue.'</textarea>';
			}
			$example = ($variable->example == '') ? '' : ' e.g. <b class="example">'.$variable->example.'</b>';
			$output .= $default.' <label for="'.$inputname.'">'.$variable->description.$example.'</label>';
		}
		else
		{
			$output = 'The variable is not a valid SetupVariable (class: '.get_class($variable).'.';
		}
		return $output;
	}
	
	function CreateInstructionOption($instruction)
	{
		$nl = "\n";
		$option = '';
		
		$input_name = 'frm_option_'.$instruction->functionName;
		if($instruction->optional)
		{
			if($_POST['setupForm_submit'] == 'Save Values')
			{
				$checked = ($_REQUEST[$input_name] == 'true') ? ' checked' : '';
			}
			else
			{
				$checked = ($instruction->selectedByDefault) ? ' checked' : '';
			}
			$option .= '<input type="checkbox" name="'.$input_name.'" value="true"'.$checked.'/>';
		}
		else
		{
			$option .= '<input type="checkbox" name="folly_'.$input_name.'" value="ignore" disabled checked />';
			$option .= '<input type="hidden" name="'.$input_name.'" value="true" />';
		}
		$option .= '<label for="'.$input_name.'">'.$instruction->description.'</label>';

		return $option;
	}
	
	function CreateMessage($class, $message)
	{
		$nl = "\n";
		return '<p class="message '.$class.'">'.$message.'</p>' . $nl;
	}
}

class SetupVariable
{
	var $name;
	var $maxlength;
	var $default;
	var $description;
	var $example;
	var $passworded;
	
	var $userValue;
	var $validationError;
	
	function SetupVariable($name, $maxlength=0, $passworded=false, $default=false, $description='', $example='')
	{
		$this->name = $name;
		$this->maxlength = $maxlength;
		$this->default = $default;
		$this->description = $description; 
		$this->example = $example;
		$this->passworded = $passworded;
	}
	
	function validate()
	{
		$this->validationError = '';
		if($this->maxlength != 0 && strlen($this->userValue) > $this->maxlength)
		{
			$this->validationError = 'Supplied value for '.$this->name.' ('.$this->$userValue.') is greater than maxlength ('.$this->maxlength.').';
			return false;
		}
		else if($this->default === false && strlen($this->userValue) == 0)
		{
			$this->validationError = 'No default value for '.$this->name.', you must supply a value: received ('.$this->userValue.').';
			return false;
		}
		return true;
	}
}

class SetupInstruction
{
	var $functionName;
	var $description;
	var $optional;
	var $selectedByDefault;
	var $executed;
	
	function SetupInstruction($functionName, $description, $optional=false, $selectedByDefault=true)
	{
		$this->functionName = $functionName;
		$this->description = $description;
		$this->optional = $optional;
		$this->selectedByDefault = $selectedByDefault;
		$this->executed = false;
	}
}

class SetupTemplate
{
	function createHTML($title='No title', $body='No body', $footer='Empty footer')
	{
		$nl = "\n";

		ob_start();
		?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<title><?php echo $title; ?></title>
<style>
<?php echo SetupTemplate::createCSS(); ?>
</style>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
</head>

<body class="setup">
<div class="layout"> 
  <div class="box body"> <?php echo $body . $nl; ?> </div>
  <div class="box actions"> <?php echo $footer . $nl; ?> </div>
</div>
</body>
</html>
		<?
		return ob_get_clean();
	}
	
	function createCSS()
	{
		ob_start();
		?>
/* Reset CSS - Rik Lomas - http://rikrikrik.com - Based on code from Nick Rigby - http://www.nickrigby.com */
*{margin:0;padding:0}iframe,a img,fieldset,form,table{border:0}h6,h5,h4,h3,h2,h1,caption,th,td{font-size:100%;font-weight:normal}dd,dt,li,dl,ol,ul{list-style:none;list-style-position:outside}legend{color:#000}button,select,textarea,input{font:100% serif}caption,th,td{text-align:left}

/* Page box layout */
body.setup {
	background: #CCC url('./background.jpg') repeat-x;
	font-family: Arial, Verdana, sans-serif;
	font-size: 1em;
	text-decoration: none;
}

.setup > .layout {
	position: relative;
	margin: auto;
	padding: 0;
	width: 900px;
	color: black;
	height: 37px;
}

.layout > .box {
	position: relative;
	border: 1px solid #CCC;
	background: white;
	color: black;
	
	margin: -1px 10px 10px 10px;
	padding: 10px;
	overflow: hidden;
}

.layout > .callout {
	position: absolute;
	top: 0;
	right: 0;
	
	width: 210px;
	margin: 0;
	
	font-size: 18px;
}
.layout > .callout > p {
	font-size: 18px;
	margin: 0 0 10px 0;
}

/* General content styles */
h1, h2 {
	font-weight: bold;
	text-align: left;
	font-size: 140%;
}
h2 {
	font-size: 110%;
	text-align: left;
	margin: 0.4em 0 1em 0;
}
h3 {
	font-size: 90%;
	margin: 0.4em 0 0.6em 0;
	font-weight: bold;
}
h4 {
	font-size: 90%;
	margin: 0.4em 0 0.6em 0;
	font-weight: bold;
	color: #333;
}
p {
	margin: 0.4em 0 1em 0;
	font-size: 80%;
}
li {
	list-style: square;
	margin: 0;
	margin-left: 1.5em;
	color: #333;
	
	font-size: 80%;
}
ol > li {
	list-style: decimal;
	padding: 2px;
}
ul {
	margin-top: 0;
	margin-bottom: 1.0 em;
}
ul ul {
	margin-top: 0;
	margin-bottom: 0;
}
a, a:hover, a:link, a:visited {
	color: #068;
	text-decoration: none;
}
table, td, tr { font-size: 12px; }
td > ul > li, td > ol > li, td > p { font-size: 100%; }
td.linebreak { overflow: hidden; border-top: 2px solid #CCC; font-size: 2px; height: 2px; }
input, textarea { font-family: Lucida Console, Courier New, Arial; padding: 2px; }
.message.warning { padding: 5px; background: #FD5; }
.message.error { padding: 5px; background: #D77; }
.message.success { padding: 5px; background: #7D7; }
label { padding: 8px; }
input.text { width: 250px; }
		<?php
		return ob_get_clean();
	}
}
?>